package java_book.ArrayBag;

import java.util.Arrays;

import java_book.bag.BagInterface;

public class ArrayBag<T> implements BagInterface<T> {
	
	private T[] bag;
	private static int CAPACITY;
	private static final int DEFAULT_CAPACITY = 25;
	private int numberOfEntries;
	
	public ArrayBag() {
		this(DEFAULT_CAPACITY);
	}
	
	public ArrayBag(int capacity) {
		numberOfEntries = 0;
		CAPACITY = capacity;
		// the cast is safe because the new array contains null entries
		@SuppressWarnings("unchecked")
		T[] tempBag = (T[])new Object[capacity];
		bag = tempBag;
	}

	@Override
	public int getCurrentSize() {
		return this.numberOfEntries;
	}

	@Override
	public boolean isFull() {
		if (numberOfEntries == CAPACITY) {
			return true;
		}
		return false;
	}

	@Override
	public boolean isEmpty() {
		if (numberOfEntries == 0) {
			return true;
		}
		return false;
	}

	@Override
	public boolean add(T newEntry) {
		int i;
		for (i = 0; i < CAPACITY; i++) {
			if (bag[i] == null) {
				ensureCapacity();
				bag[i] = newEntry;
				numberOfEntries++;
				break;
			}
		}
		return true;
	}
	
	private void ensureCapacity() {
		if (numberOfEntries == CAPACITY) {
			bag = Arrays.copyOf(bag, 2 * CAPACITY);
		}
	}

	@Override
	public T remove() {
		if (isEmpty()) {
			return null;
		}
		
		T returnItem = bag[numberOfEntries];
		numberOfEntries--;
		return returnItem;
	}

	@Override
	public boolean remove(T anEntry) {
		if (isEmpty()) {
			return false;
		}
		int i = 0;
		for (i = 0; i < CAPACITY; i++) {
			if (anEntry.equals(bag[i])) {
				bag[i] = null;
				numberOfEntries--;
				break;
			}
		}
		return true;
	}

	@Override
	public void clear() {
		int i;
		for(i = 0; i < CAPACITY; i++) {
			bag[i] = null;
		}
		numberOfEntries = 0;
	}

	@Override
	public int getFrequencyOf(T anEntry) {
		int numFreq = 0;
		for (int i = 0; i < CAPACITY; i++) {
			if (anEntry.equals(bag[i])) {
				numFreq++;
			}
		}
		return numFreq;
	}

	@Override
	public boolean contains(T anEntry) {
		int i;
		for (i = 0; i < CAPACITY; i++) {
			if (anEntry.equals(bag[i])) {
				return true;
			}
		}
		return false;
	}

	@Override
	public T[] toArray() {
		@SuppressWarnings("unchecked")
		T[] result = (T[])new Object[numberOfEntries];
		int index;
		int i = 0;
		for (index = 0; index < CAPACITY; index++) {
			if (bag[index] != null) {
				result[i] = bag[index];
				i++;
			}
		}
		return result;
	}

}
