package java_book.LinkedBag;

import java_book.bag.BagInterface;
import java_book.node.Node;

/**
 * A class of bags whose entries are stored in a chain of linked nodes.
 * The bag is never full.
 * @author mark
 *
 * @param <T>
 */
public class LinkedBag<T> implements BagInterface<T> {
	
	private Node<T> headNode;
	private Node<T> tailNode;
	private int numberOfEntries;
	
	public LinkedBag() {
		this.headNode = null;
		this.tailNode = null;
		numberOfEntries = 0;
	}

	@Override
	public int getCurrentSize() {
		return this.numberOfEntries;
	}

	@Override
	public boolean isFull() {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public boolean isEmpty() {
		if (this.numberOfEntries == 0) {
			return true;
		}
		return false;
	}

	@Override
	public boolean add(T newEntry) {
		if (headNode == null) {
			numberOfEntries++;
			headNode = new Node<T>(newEntry);
			return true;
		}
		
		Node<T> currentNode = headNode;
		while (currentNode != null) {
			if (currentNode.nextNode() == null) {
				break;
			} else {
				currentNode = currentNode.nextNode();
			}
		}
		
		if (currentNode == null) {
			System.out.println("Something happened, we received a null node");
			return false;
		}
		
		Node<T> newNode = new Node<T>(newEntry);
		currentNode.setNext(newNode);
		newNode.setPrev(currentNode);
		tailNode = newNode;
		numberOfEntries++;
		return true;
	}

	@Override
	public T remove() {
		if (headNode == null) {
			return null;
		}
		
		Node<T> returnNode = headNode;
		headNode = headNode.nextNode();
		numberOfEntries--;
		return (T) returnNode.getData();
	}

	@Override
	public boolean remove(T anEntry) {
		if (anEntry.equals(headNode.getData())) {
			headNode = headNode.nextNode();
			return true;
		}
		Node<T> currentNode = headNode;
		Node<T> prevNode;
		while (currentNode != null) {
			prevNode = currentNode.prevNode();
			if (anEntry.equals(currentNode.getData())) {
				if (prevNode != null) {
					prevNode.setNext(currentNode.nextNode());
					currentNode.nextNode().setPrev(prevNode);
					numberOfEntries--;
					return true;
				} else {
					System.out.println("Error occurred in removing");
					return false;
				}
			}
			currentNode = currentNode.nextNode();
		}
		return false;
	}

	@Override
	public void clear() {
		while (headNode != null) {
			headNode = headNode.nextNode();
		}
		numberOfEntries = 0;
	}

	@Override
	public int getFrequencyOf(T anEntry) {
		int counter = 0;
		Node<T> currentNode = headNode;
		while (currentNode != null) {
			if (anEntry.equals(currentNode.getData())) {
				counter++;
			}
		}
		return counter;
	}

	@Override
	public boolean contains(T anEntry) {
		Node<T> currentNode = headNode;
		while (currentNode != null) {
			if (anEntry.equals(currentNode.getData())) {
				return true;
			}
		}
		return false;
	}

	@Override
	public T[] toArray() {
		int index = 0;
		@SuppressWarnings("unchecked")
		T[] returnArray = (T[])new Object[numberOfEntries];
		Node<T> currentNode = headNode;
		while (currentNode != null) {
			returnArray[index] = (T) currentNode.getData();
			currentNode = currentNode.nextNode();
			index++;
		}
		return returnArray;
	}

}
