package java_book.ch11;

import java_book.queue.*;

/**
 * A class that implements a queue of objects by using an array.
 * @author mark
 *
 * @param <T>
 */
public class ArrayQueue<T> implements QueueInterface<T> {
	private T[] queue; // circular array of queue entries and one unused
					   // location
	private int frontIndex;
	private int backIndex;
	private static final int DEFAULT_INITIAL_CAPACITY = 50;
	
	public ArrayQueue() {
		this(DEFAULT_INITIAL_CAPACITY);
	}
	
	public ArrayQueue(int capacity) {
		//The cast is safe becuase the new array contains null entries
		@SuppressWarnings("unchecked")
		T[] tempQueue = (T[])new Object[capacity + 1];
		queue = tempQueue;
		frontIndex = 0;
		backIndex = capacity;
	}

	@Override
	public void enqueue(T newEntry) {
		ensureCapacity();
		backIndex = (backIndex + 1) % queue.length;
		queue[backIndex] = newEntry;
		
	}

	private void ensureCapacity() {
		if (frontIndex == ((backIndex + 2) % queue.length)) {
			T[] oldQueue = queue;
			int oldSize = oldQueue.length;
			
			// the cast is safe because the new array contains null entries
			@SuppressWarnings("unchecked")
			T[] tempQueue = (T[])new Object[2 * oldSize];
			queue = tempQueue;
			for (int index = 0; index < oldSize - 1; index++) {
				queue[index] = oldQueue[frontIndex];
				frontIndex = (frontIndex + 1) % oldSize;
			}
			
			frontIndex = 0;
			backIndex = oldSize - 2;
		}
	}

	@Override
	public T deque() {
		T front = null;
		if (!isEmpty()) {
			front = queue[frontIndex];
			queue[frontIndex] = null;
			frontIndex = (backIndex + 1) % queue.length;
		}
		return front;
	}

	@Override
	public T getFront() {
		T front = null;
		
		if (!isEmpty()) {
			front = queue[frontIndex];
		}
		return front;
	}

	@Override
	public boolean isEmpty() {
		return frontIndex == ((backIndex + 1) % queue.length);
	}

	@Override
	public void clear() {
		for (int i = 0; i < queue.length; i++) {
			queue[i] = null;
		}
		frontIndex = 0;
		backIndex = queue.length;
	}

}
