package java_book.ch11;

import java_book.queue.*;
import java_book.node.*;

/**
 * A class that implements a queue of objects by using
 * a two-part circular chain of linked nodes
 * @author mark
 *
 * @param <T>
 */
public class TwoPartCircularLinkedQueue<T> implements QueueInterface<T> {
	private Node<T> queueNode;	// references first node in queue
	private Node<T> freeNode;	// references node after back of queue
	
	public TwoPartCircularLinkedQueue() {
		freeNode = new Node<T>(null, null);
		freeNode.setNext(freeNode);
		queueNode = freeNode;
	}

	@Override
	public void enqueue(T newEntry) {
		freeNode.setData(newEntry);
		
		if (isChainFull()) {
			// allocate a new node and insert it after the node that
			// freeNode references
			Node<T> newNode = new Node<T>(null, freeNode.nextNode());
			freeNode.setNext(newNode);
		}
		
		freeNode = freeNode.nextNode();
	}

	@Override
	public T deque() {
		T front = null;
		
		if (!isEmpty()) {
			front = queueNode.getData();
			queueNode.setData(null);
			queueNode = queueNode.nextNode();
		}
		
		return front;
	}

	@Override
	public T getFront() {
		T front = null;
		
		if (!isEmpty()) {
			front = queueNode.getData();
		}
		
		return front;
	}

	@Override
	public boolean isEmpty() {
		return queueNode == freeNode;
	}
	
	public boolean isChainFull() {
		return queueNode == freeNode.nextNode();
	}

	@Override
	public void clear() {
		freeNode = new Node<T>(null, null);
		freeNode.setNext(freeNode);
		queueNode = freeNode;
		
	}

}
