package java_book.ch8;

/**
 * Class for sorting an array of Comparable objects from smallest to largest
 * @author mark
 *
 */
public class SortArray {
	
	/**
	 * Sorts the first n objects in an array into ascending order.
	 * @param a an array of Comparable objects
	 * @param n an integer > 0
	 */
	public static <T extends Comparable<? super T>> void selectionSort(T[] a, int n) {
		for (int index = 0; index < n - 1; index++) {
			int indexOfNextSmallest = getIndexOfSmallest(a, index, n - 1);
			swap(a, index, indexOfNextSmallest);
			// Assertion: a[0] <= a[1] <= ... <= a[index] <= all other
			// a[i]
		}
	}
	
	/**
	 * Sorts the first n objects in an array into ascending order
	 * @param a
	 * @param n
	 */
	public static <T extends Comparable<? super T>> void Insertion(T[] a, int first, int last) {
		for (int unsorted = first + 1; unsorted < last; unsorted++) {
			T nextToInsert = a[unsorted];
			insertInOrder(nextToInsert, a, first, unsorted - 1);
		}
	}
	
	/**
	 * Finds the index of the smallest value in a portion of an array
	 * @param a an array of Comparable objects
	 * @param first an integer >= 0 and < a.length that is the index of
	 * the first array entry to consider.
	 * @param last an integer >= first and < a.length that is the
	 * index of the last array entry to consider
	 * @return the index of the smallest value among:
	 * a[first], a[first+1]...a[last]
	 */
	private static <T extends Comparable<? super T>> int getIndexOfSmallest(T[] a, int first, int last) {
		int smallestIndex = first;
		T min = a[first];
		for (int index = first + 1; index <= last; index++) {
			if (a[index].compareTo(min) < 0) {
				min = a[index];
				smallestIndex = index;
			}
		}
		return smallestIndex;
	}
	
	/**
	 * Swaps the array entries a[i] and a[j]
	 * @param a an array of objects
	 * @param i an integer >= 0 and < a.length
	 * @param j an integer >= 0 and < a.length
	 */
	private static void swap(Object[] a, int i, int j) {
		Object temp = a[i];
		a[i] = a[j];
		a[j] = temp;
	}
	
	/**
	 * 
	 * @param anEntry
	 * @param a
	 * @param being
	 * @param end
	 */
	private static <T extends Comparable<? super T>> void insertInOrder(T anEntry, T[] a, int begin, int end) {
		int index = end;
		
		while ( (index >= begin) && (anEntry.compareTo(a[index]) < 0)) {
			a[index + 1] = a[index];
			index--;
		}
		
		a[index + 1] = anEntry;
	}

}
